@@ -2,6 +2,7 @@ use std::ffi::OsString;
 
 use clap::{
     builder::NonEmptyStringValueParser, crate_version, Arg, ArgAction, ArgMatches, Command,
+    ValueHint,
 };
 
 pub fn get_cli_arguments<'a, I, T>(args: I) -> ArgMatches
@@ -31,6 +32,7 @@ fn build_command() -> Command {
                        comparison of the respective runtimes.")
                 .required(true)
                 .action(ArgAction::Append)
+                .value_hint(ValueHint::CommandString)
                 .value_parser(NonEmptyStringValueParser::new()),
         )
         .arg(
@@ -76,6 +78,7 @@ fn build_command() -> Command {
                 .short('s')
                 .action(ArgAction::Set)
                 .value_name("CMD")
+                .value_hint(ValueHint::CommandString)
                 .help(
                     "Execute CMD before each set of timing runs. This is useful for \
                      compiling your software with the provided parameters, or to do any \
@@ -83,6 +86,24 @@ fn build_command() -> Command {
                      not every time as would happen with the --prepare option."
                 ),
         )
+        .arg(
+            Arg::new("reference")
+                .long("reference")
+                .action(ArgAction::Set)
+                .value_name("CMD")
+                .help(
+                    "The reference command for the relative comparison of results. \
+                    If this is unset, results are compared with the fastest command as reference."
+                )
+        )
+        .arg(
+            Arg::new("reference-name")
+                .long("reference-name")
+                .action(ArgAction::Set)
+                .value_name("CMD")
+                .help("Give a meaningful name to the reference command.")
+                .requires("reference")
+        )
         .arg(
             Arg::new("prepare")
                 .long("prepare")
@@ -90,6 +111,7 @@ fn build_command() -> Command {
                 .action(ArgAction::Append)
                 .num_args(1)
                 .value_name("CMD")
+                .value_hint(ValueHint::CommandString)
                 .help(
                     "Execute CMD before each timing run. This is useful for \
                      clearing disk caches, for example.\nThe --prepare option can \
@@ -98,12 +120,30 @@ fn build_command() -> Command {
                      be run prior to the corresponding benchmark command.",
                 ),
         )
+        .arg(
+            Arg::new("conclude")
+                .long("conclude")
+                .short('C')
+                .action(ArgAction::Append)
+                .num_args(1)
+                .value_name("CMD")
+                .value_hint(ValueHint::CommandString)
+                .help(
+                    "Execute CMD after each timing run. This is useful for killing \
+                     long-running processes started (e.g. a web server started in --prepare), \
+                     for example.\nThe --conclude option can be specified once for all \
+                     commands or multiple times, once for each command. In the latter case, \
+                     each conclude command will be run after the corresponding benchmark \
+                     command.",
+                ),
+        )
         .arg(
             Arg::new("cleanup")
                 .long("cleanup")
                 .short('c')
                 .action(ArgAction::Set)
                 .value_name("CMD")
+                .value_hint(ValueHint::CommandString)
                 .help(
                     "Execute CMD after the completion of all benchmarking \
                      runs for each individual command to be benchmarked. \
@@ -160,21 +200,6 @@ fn build_command() -> Command {
                      possible parameter combinations.\n"
                 ),
         )
-        .arg(
-            Arg::new("style")
-                .long("style")
-                .action(ArgAction::Set)
-                .value_name("TYPE")
-                .value_parser(["auto", "basic", "full", "nocolor", "color", "none"])
-                .help(
-                    "Set output style type (default: auto). Set this to 'basic' to disable output \
-                     coloring and interactive elements. Set it to 'full' to enable all effects \
-                     even if no interactive terminal was detected. Set this to 'nocolor' to \
-                     keep the interactive output without any colors. Set this to 'color' to keep \
-                     the colors without any interactive output. Set this to 'none' to disable all \
-                     the output of the tool.",
-                ),
-        )
         .arg(
             Arg::new("shell")
                 .long("shell")
@@ -182,6 +207,7 @@ fn build_command() -> Command {
                 .action(ArgAction::Set)
                 .value_name("SHELL")
                 .overrides_with("shell")
+                .value_hint(ValueHint::CommandString)
                 .help("Set the shell to use for executing benchmarked commands. This can be the \
                        name or the path to the shell executable, or a full command line \
                        like \"bash --norc\". It can also be set to \"default\" to explicitly select \
@@ -200,9 +226,47 @@ fn build_command() -> Command {
         .arg(
             Arg::new("ignore-failure")
                 .long("ignore-failure")
-                .action(ArgAction::SetTrue)
+                .action(ArgAction::Set)
+                .value_name("MODE")
+                .num_args(0..=1)
+                .default_missing_value("all-non-zero")
+                .require_equals(true)
                 .short('i')
-                .help("Ignore non-zero exit codes of the benchmarked programs."),
+                .help("Ignore failures of the benchmarked programs. Without a value or with \
+                       'all-non-zero', all non-zero exit codes are ignored. You can also provide \
+                       a comma-separated list of exit codes to ignore (e.g., --ignore-failure=1,2)."),
+        )
+        .arg(
+            Arg::new("style")
+                .long("style")
+                .action(ArgAction::Set)
+                .value_name("TYPE")
+                .value_parser(["auto", "basic", "full", "nocolor", "color", "none"])
+                .help(
+                    "Set output style type (default: auto). Set this to 'basic' to disable output \
+                     coloring and interactive elements. Set it to 'full' to enable all effects \
+                     even if no interactive terminal was detected. Set this to 'nocolor' to \
+                     keep the interactive output without any colors. Set this to 'color' to keep \
+                     the colors without any interactive output. Set this to 'none' to disable all \
+                     the output of the tool.",
+                ),
+        )
+        .arg(
+            Arg::new("sort")
+            .long("sort")
+            .action(ArgAction::Set)
+            .value_name("METHOD")
+            .value_parser(["auto", "command", "mean-time"])
+            .default_value("auto")
+            .hide_default_value(true)
+            .help(
+                "Specify the sort order of the speed comparison summary and the exported tables for \
+                 markup formats (Markdown, AsciiDoc, org-mode):\n  \
+                   * 'auto' (default): the speed comparison will be ordered by time and\n    \
+                     the markup tables will be ordered by command (input order).\n  \
+                   * 'command': order benchmarks in the way they were specified\n  \
+                   * 'mean-time': order benchmarks by mean runtime\n"
+            ),
         )
         .arg(
             Arg::new("time-unit")
@@ -210,8 +274,8 @@ fn build_command() -> Command {
                 .short('u')
                 .action(ArgAction::Set)
                 .value_name("UNIT")
-                .value_parser(["millisecond", "second"])
-                .help("Set the time unit to be used. Possible values: millisecond, second. \
+                .value_parser(["microsecond", "millisecond", "second"])
+                .help("Set the time unit to be used. Possible values: microsecond, millisecond, second. \
                        If the option is not given, the time unit is determined automatically. \
                        This option affects the standard output as well as all export formats except for CSV and JSON."),
         )
@@ -220,6 +284,7 @@ fn build_command() -> Command {
                 .long("export-asciidoc")
                 .action(ArgAction::Set)
                 .value_name("FILE")
+                .value_hint(ValueHint::FilePath)
                 .help("Export the timing summary statistics as an AsciiDoc table to the given FILE. \
                        The output time unit can be changed using the --time-unit option."),
         )
@@ -228,6 +293,7 @@ fn build_command() -> Command {
                 .long("export-csv")
                 .action(ArgAction::Set)
                 .value_name("FILE")
+                .value_hint(ValueHint::FilePath)
                 .help("Export the timing summary statistics as CSV to the given FILE. If you need \
                        the timing results for each individual run, use the JSON export format. \
                        The output time unit is always seconds."),
@@ -237,6 +303,7 @@ fn build_command() -> Command {
                 .long("export-json")
                 .action(ArgAction::Set)
                 .value_name("FILE")
+                .value_hint(ValueHint::FilePath)
                 .help("Export the timing summary statistics and timings of individual runs as JSON to the given FILE. \
                        The output time unit is always seconds"),
         )
@@ -245,6 +312,7 @@ fn build_command() -> Command {
                 .long("export-markdown")
                 .action(ArgAction::Set)
                 .value_name("FILE")
+                .value_hint(ValueHint::FilePath)
                 .help("Export the timing summary statistics as a Markdown table to the given FILE. \
                        The output time unit can be changed using the --time-unit option."),
         )
@@ -253,7 +321,8 @@ fn build_command() -> Command {
                 .long("export-orgmode")
                 .action(ArgAction::Set)
                 .value_name("FILE")
-                .help("Export the timing summary statistics as a Emacs org-mode table to the given FILE. \
+                .value_hint(ValueHint::FilePath)
+                .help("Export the timing summary statistics as an Emacs org-mode table to the given FILE. \
                        The output time unit can be changed using the --time-unit option."),
         )
         .arg(
@@ -272,7 +341,7 @@ fn build_command() -> Command {
             Arg::new("output")
                 .long("output")
                 .conflicts_with("show-output")
-                .action(ArgAction::Set)
+                .action(ArgAction::Append)
                 .value_name("WHERE")
                 .help(
                     "Control where the output of the benchmark is redirected. Note \
@@ -288,7 +357,11 @@ fn build_command() -> Command {
                      \n  \
                        inherit:  Don't redirect the output at all (same as '--show-output').\n\
                      \n  \
-                       <FILE>:   Write the output to the given file.",
+                       <FILE>:   Write the output to the given file.\n\n\
+                    This option can be specified once for all commands or multiple times, once for \
+                    each command. Note: If you want to log the output of each and every iteration, \
+                    you can use a shell redirection and the '$HYPERFINE_ITERATION' environment variable:\n    \
+                    hyperfine 'my-command > output-${HYPERFINE_ITERATION}.log'\n\n",
                 ),
         )
         .arg(
