@@ -3,6 +3,12 @@ use common::hyperfine;
 
 use predicates::prelude::*;
 
+/// Platform-specific I/O utility.
+/// - On Unix-like systems, defaults to `cat`.
+/// - On Windows, uses `findstr` as an alternative.
+///   See: <https://superuser.com/questions/853580/real-windows-equivalent-to-cat-stdin>
+const STDIN_READ_COMMAND: &str = if cfg!(windows) { "findstr x*" } else { "cat" };
+
 pub fn hyperfine_debug() -> assert_cmd::Command {
     let mut cmd = hyperfine();
     cmd.arg("--debug-mode");
@@ -271,6 +277,78 @@ fn can_run_failing_commands_with_ignore_failure_option() {
         .success();
 }
 
+#[cfg(unix)]
+#[test]
+fn can_ignore_specific_exit_codes() {
+    // Test that specifying exit code 1 ignores it
+    hyperfine()
+        .arg("--runs=1")
+        .arg("--ignore-failure=1")
+        .arg("exit 1")
+        .assert()
+        .success();
+
+    // Test that other exit codes still fail
+    hyperfine()
+        .arg("--runs=1")
+        .arg("--ignore-failure=1")
+        .arg("exit 2")
+        .assert()
+        .failure()
+        .stderr(predicate::str::contains(
+            "Command terminated with non-zero exit code 2",
+        ));
+}
+
+#[cfg(unix)]
+#[test]
+fn can_ignore_multiple_exit_codes() {
+    // Test that all specified exit codes are ignored
+    hyperfine()
+        .arg("--runs=1")
+        .arg("--ignore-failure=1,2,3")
+        .arg("exit 1")
+        .assert()
+        .success();
+
+    hyperfine()
+        .arg("--runs=1")
+        .arg("--ignore-failure=1,2,3")
+        .arg("exit 2")
+        .assert()
+        .success();
+
+    hyperfine()
+        .arg("--runs=1")
+        .arg("--ignore-failure=1,2,3")
+        .arg("exit 3")
+        .assert()
+        .success();
+
+    // Test that other exit codes still fail
+    hyperfine()
+        .arg("--runs=1")
+        .arg("--ignore-failure=1,2,3")
+        .arg("exit 4")
+        .assert()
+        .failure()
+        .stderr(predicate::str::contains(
+            "Command terminated with non-zero exit code 4",
+        ));
+}
+
+#[cfg(unix)]
+#[test]
+fn ignore_failure_with_all_non_zero() {
+    // Test explicit "all-non-zero" mode
+    hyperfine()
+        .arg("--runs=1")
+        .arg("--ignore-failure=all-non-zero")
+        .arg("exit 5")
+        .assert()
+        .success();
+}
+
 #[test]
 fn shows_output_of_benchmarked_command() {
     hyperfine()
@@ -306,7 +384,7 @@ fn can_pass_input_to_command_from_a_file() {
         .arg("--runs=1")
         .arg("--input=example_input_file.txt")
         .arg("--show-output")
-        .arg("cat")
+        .arg(STDIN_READ_COMMAND)
         .assert()
         .success()
         .stdout(predicate::str::contains("This text is part of a file"));
@@ -318,7 +396,7 @@ fn fails_if_invalid_stdin_data_file_provided() {
         .arg("--runs=1")
         .arg("--input=example_non_existent_file.txt")
         .arg("--show-output")
-        .arg("cat")
+        .arg(STDIN_READ_COMMAND)
         .assert()
         .failure()
         .stderr(predicate::str::contains(
@@ -493,6 +571,18 @@ fn shows_benchmark_comparison_relative_to_reference() {
         );
 }
 
+#[test]
+fn shows_reference_name() {
+    hyperfine_debug()
+        .arg("--reference=sleep 2.0")
+        .arg("--reference-name=refabc123")
+        .arg("sleep 1.0")
+        .arg("sleep 3.0")
+        .assert()
+        .success()
+        .stdout(predicate::str::contains("Benchmark 1: refabc123"));
+}
+
 #[test]
 fn performs_all_benchmarks_in_parameter_scan() {
     hyperfine_debug()
